from django.db import models


def get_categories(transactions):
    categories = (
        transactions.values("category", "category__name", "category__icon")
        .annotate(
            sum=models.Sum("value"),
            sum_m=models.Sum("value", filter=models.Q(value__lt=0)),
            sum_p=models.Sum("value", filter=models.Q(value__gt=0)),
        )
        .order_by("-sum")
    )
    return {
        "data": categories,
        "max": max(
            categories.aggregate(
                max=models.Max("sum_p", default=0),
                min=models.Min("sum_m", default=0),
            ).values(),
        ),
    }
