from django.db import models
from django.db.models import Func, Max, Min, Q, Sum, Value
from django.db.models.functions import Now, TruncMonth


class GenerateMonth(Func):
    function = "generate_series"
    template = "%(function)s(%(expressions)s, '1 month')::date"


def history(transaction_set):
    if not transaction_set.exists():
        return None
    _transaction_month = transaction_set.values(month=TruncMonth("date")).order_by(
        "-date"
    )
    _months = (
        transaction_set.values(
            month=GenerateMonth(
                _transaction_month.last()["month"],
                Now(output_field=models.DateField()),
            )
        )
        .annotate(
            sum_m=Value(0), sum_p=Value(0), sum=Value(0), has_transactions=Value(0)
        )
        .difference(
            _transaction_month.annotate(
                sum_m=Value(0), sum_p=Value(0), sum=Value(0), has_transactions=Value(0)
            )
        )
    )
    _history = _transaction_month.annotate(
        sum_p=Sum("value", filter=Q(value__gt=0)),
        sum_m=Sum("value", filter=Q(value__lt=0)),
        sum=Sum("value"),
        has_transactions=Value(1),
    ).order_by("-month")

    return {
        "data": _history.union(_months).order_by("-month"),
        "max": max(
            _history.aggregate(
                max=Max("sum_p", default=0),
                min=-Min("sum_m", default=0),
            ).values(),
        ),
    }
